capture log close
log using mco18-test-meffects-2018-04-04, replace text
version 14.2
clear all
macro drop _all
set linesize 80
set scheme s1manual

    *xmchangetocoefplot 2018-03-16 moved to xmchange 4.1.0 xmchangetocoefplot

//  Modeling Categorical Outcomes - 2018
//  Testing marginal effects from single model

//  Install coefplot package and spost13_ado

local pgm mco18-test-meffects-2018-04-04
local dte 2018-04-04
local who Scott Long
local tag "`pgm'.do `who' `dte'"

global graphfmt emf 
if "`c(os)'"!="Windows" global graphfmt pdf

local qui ""
local qui "qui"

//  =========================================================================
//  #1 tests of regression coefficients and MEs

use binlfp4, clear
logit lfp k5 k618 i.agecat i.wc i.hc lwg inc, nolog

test 1.wc

mchange wc, at(k5=0) atmeans brief
mchange wc, at(k5=1) atmeans brief
mchange wc, at(k5=2) atmeans brief
mchange wc, at(k5=3) atmeans brief

//  =========================================================================
//  #3 Plot DC and CI for all effects
    
xmchange k5 k618 agecat wc hc, /// need experimental mchange to get _mctocp
    stats(est se z pvalue ll ul) amount(1) width(7)
mat mpart1 = _mctocp // results used by coefplot

xmchange lwg inc, ///
    stats(est se z pvalue ll ul) amount(sd) width(7)
mat mpart2 = _mctocp // results used by coefplot
mat dcmat = mpart1\mpart2

local graphname "DCall-cplt"
coefplot ///
    (matrix(dcmat[,1]), ci((dcmat[,2] dcmat[,3]))) ///
    , ///
    xtitle("Average Discrete Change with 95% CI",size(*.9)) ///
    xline(0, lcolor(red*.7) lwid(*1.2)) xlabel(-.4(.2).4, grid gmax gmin) 
local out "`pgm'-`graphname'"
graph export `out'.$graphfmt, replace

//  =========================================================================
//  #4 comparing DCwc and DChc

logit lfp k5 k618 i.agecat i.wc i.hc lwg inc, nolog
estimates store logitmodel

//  DCwc and DChc

mchange hc wc, stats(est se z p ll ul) width(8)
mchange hc wc, stats(est se z p ll ul) level(90) width(8)

//  plot effects - coefplot plots resuls from margins

margins, dydx(wc hc) post // <= post!!
matlist e(b) // predictions
matlist e(V) // covariance for predictions

local graphname "DCwchc90-cplt"
coefplot, levels(90) ///
    xline(0,lcol(red*.7) lwid(*1.2)) xlabel(-.1(.1).3, grid) ///
    xtitle("Average Discrete Change with 90% CI",size(*.9)) ///
    grid(none) coeflabels(1.wc="ADC(wc)" 1.hc="ADC(hc)") ///
    ciopts(recast(rcap)) mcol(green) scale(2) ysize(2) xsize(6) 
local out "`pgm'-`graphname'"
graph export `out'.$graphfmt, replace

local graphname "DCwchc95-cplt"
coefplot, levels(95) ///
    xline(0,lcol(red*.7) lwid(*1.2)) xlabel(-.1(.1).3, grid) ///
    xtitle("Average Discrete Change with 95% CI",size(*.9)) ///
    grid(none) coeflabels(1.wc="ADC(wc)" 1.hc="ADC(hc)") ///
    ciopts(recast(rcap)) mcol(green) scale(2) ysize(2) xsize(6) 
local out "`pgm'-`graphname'"
graph export `out'.$graphfmt, replace

//  didactic
    
local graphname "DCci-overlap"
coefplot, xline(0,lcol(red*.7) lwid(*1.2)) xlabel(-.1(.1).3, grid) ///
    xtitle("Average Discrete Change",size(*.9)) ///
    grid(none) levels(95) coeflabels(1.wc="ADC(x1)" 1.hc="ADC(x2)") ///
    ciopts(recast(rcap)) mcol(green) scale(2.5) ysize(1.3) xsize(6) 
local out "`pgm'-`graphname'"
graph export `out'.$graphfmt, replace

local graphname "DCci-no-overlap"
coefplot, xline(0,lcol(red*.7) lwid(*1.2)) xlabel(-.1(.1).3, grid) ///
    xtitle("Average Discrete Change",size(*.9)) ///
    grid(none) levels(80) coeflabels(1.wc="ADC(x1)" 1.hc="ADC(x2)") ///
    ciopts(recast(rcap)) mcol(green) scale(2.5) ysize(1.3) xsize(6) 
local out "`pgm'-`graphname'"
graph export `out'.$graphfmt, replace

//  test DCwc = 0 and DChc = 0 individually : using results from margins

test 1.wc
test 1.hc

//  test DCwc = DChc = 0

test 1.wc 1.hc 

//  test DChc = DCwc

test    1.wc = 1.hc
lincom  1.wc - 1.hc
mlincom 1 - 2, stats(all)

// ==========================================================================
//  #5 comparing DCinc and DClwg for SD change

estimate restore logitmodel

//  compute effects individually to confirm later steps are correct

mchange inc lwg, stats(est se z p ll ul) amount(sd) width(8)
mchange inc lwg, stats(all) amount(sd) width(8)

//  margins to compute DCinc and DClwg

sum inc
local sdinc = r(sd) // needed from margins + SD change
sum lwg
local sdlwg = r(sd) // needed from margins + SD change

margins, at(inc=gen(inc)) at(inc=gen(inc+`sdinc')) ///
         at(lwg=gen(lwg)) at(lwg=gen(lwg+`sdlwg')) post

//  mlincom to compute DC(inc) and DC(lwg)

qui mlincom 2-1, rowname(DCinc+sd) stats(all) clear
mlincom 4-3, rowname(DClwg+sd) stats(all) add

//  second difference: DC(inc) + DC(lwg)
//
//  since the direction of the effects of income and wages differ,
//  test if sum of effects is 0

mlincom (2-1)+(4-3), rowname(2nd difference) stats(all)
scalar zvalue = r(z)

lincom (2._at-1._at)+(4._at-3._at)

//  test that DCinc = -1*DClwg 

test (2._at-1._at)=(-1*(4._at-3._at))

di "z-squared: " zvalue*zvalue

// ===========================================================================
//  #6 comparing profiles / comparing ideal types

estimates restore logitmodel // restore logit estimates

//  predictions computed individually

qui mtable, rowname(Average person) clear ci ///
    at((mean) _all) // or use atmeans
qui mtable, rowname(Young lower ed kids) below ci ///
    at(agecat=1 k5=2 k618=0 inc=10 lwg=.75 hc=0 wc=0)
qui mtable, rowname(Young higher ed kids) below ci ///
    at(agecat==1 k5==2 k618==0 wc==1 hc==1 inc=16.6 lwg=1.62)
qui mtable, rowname(Middle age higher ed kids)  below ci ///
    at(agecat==2 k5==0 k618=1.37 wc==1 hc==1 inc=27.7 lwg=1.41) 
mtable, rowname(Older higher ed) below ci ///
    at(agecat==3 k5==0 k618==0 wc==1 hc==1 inc=27.9 lwg=1.38) 

//  predictions computed simultaneously

mtable, clear ci ///
    at((mean) _all) ///
    at(agecat=1 k5=2 k618=0 inc=10 lwg=.75 hc=0 wc=0) ///
    at(agecat==1 k5==2 k618==0 wc==1 hc==1 inc=16.6 lwg=1.62) ///
    at(agecat==2 k5==0 k618=1.37 wc==1 hc==1 inc=27.7 lwg=1.41) ///
    at(agecat==3 k5==0 k618==0 wc==1 hc==1 inc=27.9 lwg=1.38) ///
    post

//  test differences in profile probabilities

mlincom 4 - 5, rowname(MidEdDad-OldHiEd) clear twidth(20)
qui {
mlincom 1 - 2, rowname(Average-YngLoEdMom) clear
mlincom 1 - 3, rowname(Average-YngHiEdMom) add
mlincom 1 - 4, rowname(Average-MidEdDad) add
mlincom 1 - 5, rowname(Average-OldHiEd) add
mlincom 2 - 3, rowname(YngLoEdMom-YngHiEdMom) add
mlincom 2 - 4, rowname(YngLoEdMom-MidEdDad) add
mlincom 2 - 5, rowname(YngLoEdMom-OldHiEd) add
mlincom 3 - 4, rowname(YngHiEdMom-MidEdDad) add
mlincom 3 - 5, rowname(YngHiEdMom-OldHiEd) add
}
mlincom 4 - 5, rowname(MidEdDad-OldHiEd) add twidth(25)
est restore logitmodel

// ===========================================================================
//  #7 comparing profiles using r(atspec)

estimates restore logitmodel // restore logit estimates

//  example of r(atspec)

mtable, atmeans
    local Average "`r(atspec)'" // atspec used by mtable
di "`Average'"    

mtable, at(agecat=1 k5=2 k618=0 inc=10 lwg=.75 hc=0 wc=0)
    local YoungLowEdMom "`r(atspec)'"
di "`YoungLowEdMom'"

mtable, at(`Average') at(`YoungLowEdMom') 

//  get atspecs for five profiles

quietly{
    mtable, rowname(Average person) atmeans clear ci
    local Average "`r(atspec)'"

    mtable, rowname(Young lower ed kids) ///
        at(agecat=1 k5=2 k618=0 inc=10 lwg=.75 hc=0 wc=0) below ci
    local YoungLowEdMom "`r(atspec)'"

    sum lwg if agecat==1 & k5==2 & k618==0 & wc==1 & hc==1
        local mnlwg = r(mean)
    sum inc if agecat==1 & k5==2 & k618==0 & wc==1 & hc==1
        local mninc = r(mean)
    mtable, at(agecat==1 k5==2 k618==0 wc==1 hc==1 inc=`mninc' lwg=`mnlwg') ///
        rowname(Young higher ed kids) atmeans below ci
    local YoungHiEdMom "`r(atspec)'"

    sum inc if agecat==2 & k5==0 & wc==1 & hc==1
        local mninc = r(mean)
    sum lwg if agecat==2 & k5==0 & wc==1 & hc==1
        local mnlwg = r(mean)
    sum k618 if agecat==2 & k5==0 & wc==1 & hc==1
        local mnlk618 = r(mean)
    mtable, at(agecat==2 k5==0 k618=`mnlk618' ///
        wc==1 hc==1 inc=`mninc' lwg=`mnlwg') ///
        rowname(Middle age higher ed kids) atmeans ci below
    local MiddleEdDad "`r(atspec)'"

    sum inc if agecat==3 & wc==1 & hc==1 & k618==0 & k5==0
        local mninc = r(mean)
    sum lwg if agecat==3 & wc==1 & hc==1 & k618==0 & k5==0
        local mnlwg = r(mean)
    mtable , at(agecat==3 k5==0 k618==0 wc==1 hc==1 inc=`mninc' lwg=`mnlwg') ///
        rowname(Older higher ed) atmeans ci below
    local OlderHiEd "`r(atspec)'"
}

//  joint estimation with atspecs for five profiles

mtable, at(`Average')       at(`YoungLowEdMom') at(`YoungHiEdMom') ///
        at(`MiddleEdDad')   at(`OlderHiEd')     post ci

//  test differences

qui {
mlincom 1 - 2, rowname(Average-YngLoEdMom) clear
mlincom 1 - 3, rowname(Average-YngHiEdMom) add
mlincom 1 - 4, rowname(Average-MidEdDad) add
mlincom 1 - 5, rowname(Average-OldHiEd) add
mlincom 2 - 3, rowname(YngLoEdMom-YngHiEdMom) add
mlincom 2 - 4, rowname(YngLoEdMom-MidEdDad) add
mlincom 2 - 5, rowname(YngLoEdMom-OldHiEd) add
mlincom 3 - 4, rowname(YngHiEdMom-MidEdDad) add
mlincom 3 - 5, rowname(YngHiEdMom-OldHiEd) add
}
mlincom 4 - 5, rowname(MidEdDad-OldHiEd) add twidth(25)

log close
exit

// ===========================================================================
// #20 Beta and DC tests

qui logit lfp k5 k618 i.agecat i.wc i.hc lwg inc, nolog
mat beta = r(table)'
mat beta = beta[1...,1], beta[1...,4]
mat beta = beta[1..2,1...] \ beta[4..5,1...] \ beta[7,1...]  \ beta[9..11,1...] 
mat colnames beta = Beta:est Beta:prob

xmchange, amount(marginal) atmeans brief
mat dcm = r(table)
mat dcm = dcm[1..4,1...] \ dcm[6...,1...]
mat colnames dcm = DCmean:est DCmean:prob

xmchange, amount(marginal) at(inc=0) atmeans brief
mat dcm0 = r(table)
mat dcm0 = dcm0[1..4,1...] \ dcm0[6...,1...]
mat colnames dcm0 = DCinc0:est DCinc0:prob
matlist dcm0

xmchange, amount(marginal) at(inc=100) atmeans brief
mat dcm100 = r(table)
mat dcm100 = dcm100[1..4,1...] \ dcm100[6...,1...]
mat colnames dcm100 = DCinc100:est DCinc100:prob

mat betadc = beta, dcm, dcm0, dcm100
set linesize 90
matlist betadc, format(%5.2f)
set linesize 80

// ===========================================================================
// #30 comparing DCwc and DChc

estimate restore logitmodel

xmchange hc wc, stats(est se z p ll ul) width(8)
xmchange hc wc, stats(est se z p ll ul) width(8) details

// #31 DCwc and DChc

margins, dydx(wc hc) post // <= post!!
matlist e(b) // predictions
matlist e(V) // covariance for predictions
margins, coeflegend

// #32 test DCwc = 0 and DChc = 0

test 1.wc
test 1.hc

// #33 test DCwc = DChc = 0

test 1.wc 1.hc // not test wc hc

//  #34 DChc = DCwc

test   1.wc = 1.hc
lincom 1.wc - 1.hc
mlincom   1 - 2, stats(all)

// ===========================================================================
// #40 comparing DCinc and DClwg

estimate restore logitmodel

sum inc
    local sdinc = r(sd)
sum lwg
    local sdlwg = r(sd)

// #41 DCinc=0 and DClwg=0

xmchange inc lwg, stats(est se z p ll ul) width(8) details

// #42 margins to compute DCinc

margins, at(inc=gen(inc)) at(inc=gen(inc+`sdinc')) post
matlist e(b)
matlist e(V)

margins, coeflegend

test 1._at
test 1._at = 2._at
lincom 1._at - 2._at
mlincom 1 - 2

// #43 estimate DCinc=0 and DClwg=0

estimate restore logitmodel

margins, at(inc=gen(inc)) at(inc=gen(inc+`sdinc')) ///
         at(lwg=gen(lwg)) at(lwg=gen(lwg+`sdlwg')) post

// #44 test DCinc = 0 and DClwg = 0

test 1._at
test 2._at

// #45 test DCinc = DClwg = 0

test 1._at 2._at 

// #46 DCinc = DClwg

test    1._at = 2._at
lincom  1._at - 2._at
mlincom 1 - 2, stats(all)

est restore logitmodel

// ===========================================================================
// #50 testing four effects

estimate restore logitmodel

sum inc
    local sdinc = r(sd)
sum lwg
    local sdlwg = r(sd)

// #51 individual effects

xmchange hc wc inc lwg, stats(est se z p ll ul) width(8) details

// #52 margins to simultaneoulys estimate multiple effects

estimate restore logitmodel

margins, at(inc=gen(inc)) at(inc=gen(inc+`sdinc')) ///
         at(lwg=gen(lwg)) at(lwg=gen(lwg+`sdlwg')) ///
         at(wc=0) at(wc=1) ///
         at(hc=0) at(hc=1) ///
         post

// #53 test individually

test 2._at=1._at
test 4._at=3._at
test 6._at=5._at
test 8._at=7._at

// #54 test all are equal

test (2._at=1._at) (4._at=3._at) (6._at=5._at) (8._at=7._at)

est restore logitmodel

//  TODO: Add comparing profiles





NOT USED
//  =========================================================================
//  #1 tests of regression coefficients and MEs

//  fit model and test beta's for agecat

use binlfp4, clear
logit lfp k5 k618 i.agecat i.wc i.hc lwg inc, nolog
estimates store logitmodel
mat beta = r(table)' // save betas to matrix
testparm i.agecat // total effect of agecat

//  marginal changes 

mchange, amount(marginal) atmeans brief // atmeans
mat MEmean = r(table)

* ME's at inc=0
mchange, amount(marginal) at(inc=0) atmeans brief
mat MEat0 = r(table)

* ME's at inc=0
mchange, amount(marginal) at(inc=100) atmeans brief
mat MEat100 = r(table)

//  ADVANCED: combine and print betas and MEs

mat beta = beta[1...,1], beta[1...,4]
mat beta = beta[1..2,1...] \ beta[4..5,1...] ///
         \ beta[7,1...]    \ beta[9..11,1...] 
mat colnames beta = Beta:est Beta:prob

mat MEmean = MEmean[1..4,1...] \ MEmean[6...,1...]
mat colnames MEmean = MEmean:est MEmean:prob

mat MEat0 = MEat0[1..4,1...] \ MEat0[6...,1...]
mat colnames MEat0 = DCinc0:est DCinc0:prob

mat MEat100 = MEat100[1..4,1...] \ MEat100[6...,1...]
mat colnames MEat100 = DCinc100:est DCinc100:prob

mat betame = beta, MEmean, MEat0, MEat100
set linesize 90
matlist betame, format(%5.2f)
set linesize 80
