capture log close
log using mco18-brm-lfp-2018-04-03, replace text
version 14.2
clear all
macro drop _all
set linesize 80
set scheme s1manual

//  Modeling Categorical Outcomes - 2018
//  Binary logit and probit with Mroz data

local pgm mco18-brm-lfp-2018-04-03
local dte 2018-04-03
local who Scott Long
local tag "`pgm'.do `who' `dte'"

global graphfmt emf 
if "`c(os)'"!="Windows" global graphfmt pdf

//  #1  load data and descriptive statistics

use binlfp4, clear
codebook lfp k5 k618 agecat wc hc lwg inc, compact
tab1 agecat, miss

//  #2 fit logit and probit and compare coefficients

logit lfp k5 k618 i.agecat i.wc i.hc lwg inc
estimates store fullmodel

probit lfp k5 k618 i.agecat i.wc i.hc lwg inc
estimates store probitmodel 

estimates table fullmodel probitmodel, ///
    stats(aic bic r2_p ll) b(%8.3f) t(%8.3f) p(%8.3f)

//  #3 in-sample predictions

estimates restore probitmodel
predict prprobit
label var prprobit "Probit: predicted probabilty"

estimates restore fullmodel
predict prlogit
label var prlogit "Logit: predicted probabilty"

//  comparing logit and probit

pwcorr prprobit prlogit
local graphname prlogitVprprobit
twoway scatter prlogit prprobit, xsize(7) ysize(7) ///
    mcol(blue) msym(Oh) ///
    xlab(0(.2)1, grid gmin gmax) ylab(0(.2)1, grid gmin gmax) ///
    caption("`graphname' `tag'",size(*.5) pos(5) col(gs10)) aspect(1)
graph export `pgm'-`graphname'.$graphfmt, replace
    
//  dotplot of probit predictions

qui sum prlogit // compute mean to include in graph
local mn = string(r(mean),"%5.3f") // store formatted string

local graphname "fullmodel-phat"
dotplot prlogit, ///
    ylab(0(.2)1, nogrid) ylin(0 1, lcol(blue)) mcol(gs10) ///
    title(Model: logit lfp k5 k618 i.agecat i.wc i.hc lwg inc, pos(11)) ///
    subtitle("Observed proportion of 1's: `mn'", pos(11)) ///
    caption("`graphname' `tag'",size(*.5) pos(5) col(gs10))
graph export `pgm'-`graphname'.$graphfmt, replace

//  #4  marginal effects 
//  Options: amount(one sd) stats(est p ll ul)

est restore fullmodel
mchange, atmeans 

//  #4.1 DCM

mchange, atmeans amount(one sd) dec(2)
mchange, atmeans amount(one sd) stats(est p ll ul) dec(2)

//  over range without and with trimming

mchange lwg inc, atmeans amount(range) dec(2) brief
mchange lwg inc, atmeans amount(range) trim(5) dec(2) brief 

//  from 0 to 1
mchange k5, at(k5=0) delta(1) amount(delta) ///
    stat(from to change pvalue) atmeans dec(2)

//  #4.2 ADC: no atmeans

mchange, amount(one sd) stats(est p ll ul) dec(2)

mchange k5 wc, amount(one sd) stats(est p ll ul) dec(2)

mchange k5, at(k5=0) delta(3) amount(delta) ///
    stat(from to change pvalue) dec(2) 

//  #5 distribution of effects

    * See mco18-brm-dc-distribution-*.do

//  #6 ideal types - see compare effects do-file for testing equality

estimates restore fullmodel

//  #6.1 start table with average person

mtable, rowname(Average person) atmeans clear ci

//  #6.2 Young_Lower_Mom: young, lower class, low ed, mother
//  In 1975 $2.10 is min wage; .75 for lwg

mtable, rowname(Younger lower educ w kids) ///
    at(agecat=1 k5=2 k618=0 inc=10 lwg=.75 hc=0 wc=0) below ci twidth(28)

//  #6.3 YoungHiEdMom: young, middle class, higher ed, mother

sum lwg if agecat==1 & k5==2 & k618==0 & wc==1 & hc==1
    local mnlwg = r(mean)
sum inc if agecat==1 & k5==2 & k618==0 & wc==1 & hc==1
    local mninc = r(mean)
    
mtable, at(agecat==1 k5==2 k618==0 wc==1 hc==1 inc=`mninc' lwg=`mnlwg') ///
    rowname(Young more educ w kids) atmeans below ci twidth(28)

//  #6.4  Middle age higher ed kids |    0.748     0.672     0.823

sum inc if agecat==2 & k5==0 & wc==1 & hc==1
    local mninc = r(mean)
sum lwg if agecat==2 & k5==0 & wc==1 & hc==1
    local mnlwg = r(mean)
sum k618 if agecat==2 & k5==0 & wc==1 & hc==1
    local mnlk618 = r(mean)

mtable, at(agecat==2 k5==0 k618=`mnlk618' ///
           wc==1 hc==1 inc=`mninc' lwg=`mnlwg') ///
    rowname(Midage higher educ w kids) atmeans ci below twidth(28)

//  #6.5 Older higher ed

sum inc if agecat==3 & wc==1 & hc==1 & k618==0 & k5==0
    local mninc = r(mean)
sum lwg if agecat==3 & wc==1 & hc==1 & k618==0 & k5==0
    local mnlwg = r(mean)

mtable , at(agecat==3 k5==0 k618==0 wc==1 hc==1 inc=`mninc' lwg=`mnlwg') ///
    rowname(Older w higher educ) atmeans ci below  twidth(28)

//  #7 table of predictions

//  #7.1 table with predictions for wc by k5 using atmeans

mtable, atmeans at(wc=(0 1) k5=(0 1 2 3))

// #7.2  plot of curves leading to predictions

mgen, atmeans at(wc=0 k5=(0 1 2 3)) stub(wc0) // points
mgen, atmeans at(wc=1 k5=(0 1 2 3)) stub(wc1)

mgen, atmeans at(wc=0 k5=(-4(.1)4)) stub(xwc0) // lines
mgen, atmeans at(wc=1 k5=(-4(.1)4)) stub(xwc1)

label var xwc0pr "Did Not Attend College"
label var xwc1pr "Attended College"
local ytitle "Pr( In Labor Force )"
local line1 "msym(i) mcol(red) lcol(green) lpat(solid)"
local line0 "msym(i) mcol(red) lcol(red)   lpat(dash)"

local graphname "k5wcgraph"
graph twoway ///
    (connected xwc1pr xwc0k5, `line1' `GElw' )  ///
    (scatter   wc1pr wc0k5, msym(O) msiz(*1.3) mcol(green)) ///
    (scatter   wc0pr wc0k5, msym(S) msiz(*1.1) mcol(red)) ///
    (connected xwc0pr xwc0k5, `line0'), ///
    xtitle("Number of Children") ytitle("`ytitle'") ///
    xlabel(-4(1)4) ylabel(0(.25)1., grid gmin gmax) ///
    xline(0 1 2 3 4, lcolor(gs10)) legend(rows(1) order(1 4)) ///
    caption("`graphname' `tag'", size(*.5) pos(5) col(gs10))
graph export `pgm'-`graphname'.emf, replace

//  #7.3 means conditional on k5 using mtable

mtable if k5==0, atmeans estname(k5_0) at(wc=(0 1) k5=0) atvars(1.wc)
    // atmeans based on sample with k5==0
mtable if k5==1, atmeans estname(k5_1) at(wc=(0 1) k5=1) atvars(_none) ///
    right // atmeans based on sample with k5==1
mtable if k5==2, atmeans estname(k5_2) at(wc=(0 1) k5=2) atvars(_none) ///
    right // atmeans based on sample with k5==2
mtable if k5==3, atmeans estname(k5_3) at(wc=(0 1) k5=3) atvars(_none) ///
    right // atmeans based on sample with k5==3

//  #7.4 dc(wc|k5=j) wc using local means

mtable if k5==0, dydx(wc) stat(est p) atmeans clear long ///
    roweqnm(DCwc) coleqnm(k5_0)
mtable if k5==1, dydx(wc) stat(est p) atmeans right long coleqnm(k5_1)
mtable if k5==2, dydx(wc) stat(est p) atmeans right long coleqnm(k5_2)
mtable if k5==3, dydx(wc) stat(est p) atmeans right long coleqnm(k5_3)

//  #7.5 local means using over (add details for more information)

mtable, atmeans at(wc=(0 1) k5=0) atvars(1.wc) over(k5)

//  #7.6 construct a cleaner table with dc for wc

mtable, atmeans at(wc=(0) k5=(0 1 2 3)) atvars(k5) ///
    clear estname(NoCol)
mtable, atmeans at(wc=(1) k5=(0 1 2 3)) atvars(_none) ///
    right estname(College)
mtable, atmeans dydx(wc) at(k5=(0 1 2 3)) atvars(_none) /// DCwc
    right estname(Diff) stats(est p)

//  #8 set options for graphs

local labYopt  "labsiz(*1.1) glwid(*.7) glcol(black*.3) grid gmin gmax"
local labXopt  "labsiz(*1.1) glwid(*.7) glcol(black*.3) nogrid"
local titleopt "ring(2) pos(11) size(*1)"
local lin1opt  "lcol(blue*1.) lpat(solid) msym(i)  msiz(*1.) mcol(blue*1.)"
local lin2opt  "lcol(red*1.) lpat(dash) msym(i)  msiz(*.9) mcol(red*1.)"
local linDCopt "lcol(green*1.) lpat(solid) msym(i)  msiz(*.9) mcol(green*1.)"
local linLOWopt "msym(i) lcol(blue)  lpat(dash)"
local linPRopt  "msym(i) lcol(green) lpat(solid)"   

//  #9 lowess plots to evaluate functional form

//  #9.1 quick lowess

local graphname lowess-inc-quick
lowess lfp inc
graph export `pgm'-`graphname'.$graphfmt, replace

//  #9.2 lowess with options to make cleaner graph

sort inc
local graphname lowess-inc
lowess lfp inc, jitter(3) generate(lowesslfp) bwidth(.5) ///
    msym(oh) lineopt(lcol(blue) lwid(*1.3)) ///
    xlab(0(20)100) ytitle(Smoothed mean LFP) ///
    ylab(0(.25)1., grid gmin gmax) yline(0 1, lcol(gs13)) ///
    caption("`graphname' `tag'", size(*.5) pos(5) col(gs10))
graph export `pgm'-`graphname'.$graphfmt, replace
label var lowesslfp "Lowess"

// if you have many observations, add nograph option

//  #9.2 comparing lowess to predictions from model

logit lfp inc // with only income

//  generate predictions
//  - you can't combine lowess with marginsplot so must use mgen

mgen, at(inc=(0(5)100)) atmeans stub(PLT) predlabel(Logit prediction) replace 
label var PLTpr "Logit prediction"

//  combining lowess with predictions

local graphname lowess-inc-phat-mgen
graph twoway ///
    (connected PLTpr PLTinc,  `linPRopt')  /// from mgen
    (connected lowesslfp inc, `linLOWopt'), /// from lowess command
    subtitle("Model including only income", position(11)) ///
    xtitle("Family income excluding wife's") ///
    ytitle("LFP") xlab(0(20)100) ylab(0(.25)1., grid gmin gmax) ///
    yline(0 1, lcol(gs13)) legend(rows(1)) ///
    caption("`graphname' `tag'", size(*.5) pos(5) col(gs10)) scale(1.2)
graph export `pgm'-`graphname'.$graphfmt, replace

//  combining lowess with predictions adding a CI

local graphname lowess-inc-phat-ci-mgen
graph twoway ///
    (rarea PLTul PLTll PLTinc, color(black*.1)) /// shaded CI
    (connected PLTpr PLTinc, `linPRopt') /// line for prob
    (connected lowesslfp inc, `linLOWopt'), ///
    subtitle("Model including only income", position(11)) ///
    ytitle("Pr(In Labor Force)'") ylab(0(.25)1., grid gmin gmax) ///
    xtitle("Family income excluding wife's") legend(off) ///
    caption("`graphname' `tag'", size(*.5) pos(5) col(gs10)) scale(1.2)
graph export `pgm'-`graphname'.$graphfmt, replace

//  #10 marginsplot plot of predicted probabilities

//  #10.1 lfp with only income

logit lfp inc 

margins, at(inc=(0(5)100)) // predictions to plot

//  quick and dirty marginsplot

local graphname lfp-inconly-quick-mplt
marginsplot, ///
    caption("`graphname' `tag'", size(*.5) pos(5) col(gs10)) scale(1.1) 
graph export `pgm'-`graphname'.$graphfmt, replace

//  customized marginsplot

local graphname lfp-inconly-mplt
marginsplot, recastci(rarea) ciopts(color(black*.1)) ///
    title("Income is the only regressor" " ", `titleopt') ///
    ylab(0(.25)1, `labYopt') xlab(0(20)100, `labXopt') ///
    plot1opts(`lin1opt') plotopts(lwidth(*1.2)) ///
    xtitle("Family Income Excluding Wife") ytitle("Pr(In Labor Force)") ///
    caption("`graphname' `tag'", size(*.5) pos(5) col(gs10)) scale(1.1)
graph export `pgm'-`graphname'.$graphfmt, replace

//  #10.2 full model

estimates restore fullmodel
margins, at(inc=(0(5)100)) atmeans

//  quick plot

local graphname lfp-inc-quick-mplt
marginsplot, ///
    caption("`graphname' `tag'", size(*.5) pos(5) col(gs10)) scale(1.1) 
graph export `pgm'-`graphname'.$graphfmt, replace

//  formatted plot

local graphname lfp-inc-mplt
marginsplot, recastci(rarea) ciopts(color(black*.1)) ///
    ylab(0(.25)1, `labYopt') xlab(0(20)100, `labXopt') ///
    plot1opts(`lin1opt') plotopts(lwidth(*1)) /// 
    xtitle("Family Income Excluding Wife") ytitle("Pr(In Labor Force)") ///
    title("Other variables at their means" " ", `titleopt') ///
    caption("`graphname' `tag'", size(*.5) pos(5) col(gs10)) scale(1.1)
graph export `pgm'-`graphname'.$graphfmt, replace

//  #10.3 predictions at two levels of wc

margins, at(inc=(0(5)100) wc=(0 1)) atmeans

local graphname lfp-incXwc-quick-mplt
marginsplot, ///
    caption("`graphname' `tag'", size(*.5) pos(5) col(gs10)) scale(1.1) 
graph export `pgm'-`graphname'.$graphfmt, replace

local graphname lfp-incXwc-mplt
local legendopt "ring(0) pos(1) rows(2)"
marginsplot, ///
    recastci(rarea) ci1opts(color(black*.2)) ci2opts(color(black*.1)) ///
    ylab(0(.25)1, `labYopt') xlab(0(20)100, `labXopt') ///
    legend(order(4 "Attended college" 3 "Did not attend") `legendopt') ///
    plot1opts(`lin1opt') plot2opts(`lin2opt') plotopts(lwidth(*1.2)) ///
    xtitle("Family Income Excluding Wife") ytitle("Pr(In Labor Force)") ///
    title("Other variables at their means" " ", `titleopt') ///
    caption("`graphname' `tag'", size(*.5) pos(5) col(gs10)) scale(1.1) 
graph export `pgm'-`graphname'.$graphfmt, replace

//  #10.4 DC(wc|income, atmeans)

margins, dydx(wc) at(inc=(0(5)100)) atmeans

local graphname lfp-incDCwc-mplt
marginsplot, recastci(rarea) ciopts(color(black*.1)) ///
    ylab(0(.1).4, `labYopt') xlab(0(20)100, `labXopt') ///
    plot1opts(`linDCopt') plotopts(lwid(*1)) ///
    xtitle("Family Income Excluding Wife") ///
    ytitle("Pr(LFP|WC=1) - Pr(LFP|WC=0)") ///
    title("Other variables at their means" " ", `titleopt') ///
    caption("`graphname' `tag'", size(*.5) pos(5) col(gs10)) scale(1.1)
graph export `pgm'-`graphname'.$graphfmt, replace

//  #11 plots using mgen

//  locals used with mgen and to construct graohs

local linDCopt  msym(i) lcol(green) lpat(solid) 
local linPR1opt msym(i) lcol(red)   lpat(dash)
local linPR0opt msym(i) lcol(blue)  lpat(solid)

//  #11.1 predictions at two levels of wc
//
//  1.  Compute predicted probabilities by income for two levels of wc.
//  2.  Plot DC(wc) 

//  predictions for income at two levels of wc

estimates restore fullmodel

mgen, at(inc=(0(5)100) wc=0) atmean stub(PLTwc0) ///
    predlabel(Did not attend college)
label var PLTwc0pr "Did not attend college"

mgen, at(inc=(0(5)100) wc=1) atmean stub(PLTwc1) ///
    predlabel(Attended college)
label var PLTwc1pr "Attended college"

local gnm full-incXwc-mgen
graph twoway ///
    (rarea PLTwc1ul PLTwc1ll PLTwc1inc, color(black*.1)) ///
    (rarea PLTwc0ul PLTwc0ll PLTwc0inc, color(black*.2)) ///
    (connected PLTwc1pr PLTwc1inc, `linPR1opt') ///
    (connected PLTwc0pr PLTwc0inc, `linPR0opt'), ///
    xtitle("Family income excluding wife's") ///
    ytitle("Pr(In Labor Force)") ylab(0(.25)1., grid gmin gmax) ///
    subtitle("Regressors: k5 k618 agecat wc hc lwg inc", position(11)) ///
    legend(on order(3 4)) ///
    caption("`gnm' `tag'",size(vsmall) pos(5) col(gs10)) scale(1.2)
graph export `pgm'-`gnm'.$graphfmt, replace

//  #8.4 discrete change for wc by inc

mgen, dydx(wc) at(inc=(0(5)100)) atmean stub(PLTdc) ///
    predlabel(DC of wc by income)

local gnm full-incXwc-DC-mgen
graph twoway ///
    (rarea PLTdcul PLTdcll PLTdcinc, color(black*.1)) /// shaded CI
    (connected PLTdcd_pr1 PLTdcinc, `linDCopt'), ///
    xtitle("Family income excluding wife's") ///
    ytitle("Pr(LFP|WC=1) - Pr(LFP|WC=0)") ylab(0(.1).4, grid gmin gmax) ///
    subtitle("Regressors: k5 k618 agecat wc hc lwg inc", position(11)) ///
    legend(off) ///
    caption("`gnm' `tag'",size(vsmall) pos(5) col(gs10)) scale(1.2)
graph export `pgm'-`gnm'.$graphfmt, replace


//  #12 odds ratios

//  #12.1 didactic -- for making a choice

global cost1 = .45
global cost2 = .51
di "Dif   = $cost2 - $cost1 = " $cost2 - $cost1
di "Ratio = $cost2 / $cost1 = " $cost2 / $cost1
di "OR    = ($cost1/(1-$cost1)) / ($cost2/(1-$cost2)) = " ///
    ($cost2/(1-$cost2)) / ($cost1/(1-$cost1))

//  #12.2 odds ratios

estimates restore fullmodel
logit
logit, or
listcoef, constant help
listcoef, constant percent help

log close
exit

 DROPPED

//  #4.5b using earlier versions of Stata

gen wc_orig = wc // save original values for wc
label var wc_orig "Hold original wc"

//  compute DC of wc for each observation

replace wc = 0 // make all cases wc=0 and then predict
predict effects_phat0

replace wc = 1 // make all cases wc=1 and then predict
predict effects_phat1

replace wc = wc_orig // restore original values
drop wc_orig

gen double me_wc = effects_phat1 - effects_phat0 // DCwc
label var me_wc "Discrete change of wc on Pr(LFP)"

* exactly the same results
sum me_wc dydxwc2

//  #4.1 extreme predictions

sort prlogit
format %9.3g prlogit lfp k5 agecat wc lwg wages inc
list prlogit lfp k5 agecat wc wages inc if prlogit<.10, clean
list prlogit lfp k5 agecat wc wages inc if prlogit>.90, clean


