capture log close
log using mco18-cme-stack-suest-2018-04-10, replace text
version 14.2
clear all
macro drop _all
set linesize 80
set scheme s1manual

//  Modeling Categorical Outcomes - 2018
// CME: Comparing Marginal Effects Across Models
// Stacking and SUEST estimation

local pgm mco18-cme-stack-suest-2018-04-10  
local dte 2018-04-10
local who Scott Long
local tag "`pgm'.do `who' `dte'"

global graphfmt emf 
if "`c(os)'"!="Windows" global graphfmt pdf

//  ===============================================================
//  #1 Data management

use addhealth4, clear
gen id = _n
* simpler names
clonevar depress = depB
clonevar sick = badhealthB
clonevar roles = role

//  ===============================================================
//  #2 fit models separatel;y

logit depress roles, nolog
est store m1

logit sick roles, nolog
est store m2

//  show robust standard errors to compare with suest

logit depress roles, nolog vce(robust)
est store m1robust

logit sick roles, nolog vce(robust)
est store m2robust

//  ===============================================================
//  #3 suest estimation

suest m1 m2
est store suest

//  ===============================================================
//  #4 create stacked dataset

clonevar rolesm1 = role
lab var rolesm1 "roles used for model 1"
clonevar rolesm2 = role
lab var rolesm1 "roles used for model 2"

gen zero = 0 
lab var zero "always 0"
gen one = 1 
lab var one "always 1"
gen two = 2 
lab var two "always 2"

stack ///
    id depress  rolesm1  zero       one     /// model 1 observations
    id sick     zero     rolesm2    two,    /// model 2 observations
    into(id y   rolesm1  rolesm2    model) clear  

gen ismodel2 = (model==2) // 1 if model 2
lab var ismodel2 "1 if observation for fitting model 2"

//  ===============================================================
//  #5 stacked estimation

//  #5a using stacked data for separate models

logit y rolesm1 if model==1
logit y rolesm2 if model==2

//  #5b joint estimation

logit y ismodel2 rolesm1 rolesm2, vce(cluster id)
est store stack
mat v = e(V) // store covariance matrix for later use

//  #5c test equal betas

lincom _b[rolesm1] - _b[rolesm2]

//  #5d view hand computations 

matlist e(V), format(%15.12f)
matlist e(b), format(%15.12f)

//  #5e hand computations of z-test

scalar b1 = _b[rolesm1]
scalar b2 = _b[rolesm2]
scalar se1 = _se[rolesm1]
scalar se2 = _se[rolesm2]
scalar cov12 = el(v,3,2)

//  correct z-statistics using cross-model covariance

di (b1-b2)/sqrt(se1^2 + se2^2 - 2*cov12)

//  incorrect z-statistics without cross-model covariance

di (b1-b2)/sqrt(se1^2 + se2^2 )

//  demonstrate computations with different precision 

di (-0.200629741713 - -0.260767812043) / sqrt(0.000660617586 + 0.001375530240 -2*0.000193613550)
di (-0.200629741713 - -0.260767812043) / sqrt(0.000660617586 + 0.001375530240)

di (-0.2006297417 - -0.2607678120) / sqrt(0.0006606176 + 0.0013755302 -2*0.0001936136)
di (-0.2006297417 - -0.2607678120) / sqrt(0.0006606176 + 0.0013755302)

di (-0.200630 - -0.260768) / sqrt(0.000661 + 0.001376 -2*0.000194)
di (-0.200630 - -0.260768) / sqrt(0.000661 + 0.001376)

di (-0.20063 - -0.26077) / sqrt(0.00066 + 0.00138 -2*0.00019)
di (-0.20063 - -0.26077) / sqrt(0.00066 + 0.00138)

//  ===============================================================
//  #6 compare estimation results results

est table _all, b(%8.5f) se(%8.7f)

est restore stack
lincom _b[ismodel2] + _b[_cons] // m2 constant

//  ===============================================================
//  #7 table

esttab m1 m2 stack

log close
exit

