capture log close
log using mco18-gme-2018-04-11, replace text
version 14.2
clear all
macro drop _all
set linesize 80
set scheme s1manual

//  Modeling Categorical Outcomes - 2018
//  Generalized marginal effects

local pgm mco18-gme-2018-04-11
local dte 2018-04-11
local who Scott Long
local tag "`pgm'.do `who' `dte'"

global graphfmt emf 
if "`c(os)'"!="Windows" global graphfmt pdf

//  globals for graphs

global TTLDCaddS ///
`"{&Delta}Pr/{&Delta}(weight{&rarr}weight+25)"'

global TTLdcpctS ///
`"{&Delta}Pr/{&Delta}(weight{&rarr}1.14*weight)"'

global TTLdpr_dage {&Delta}Pr(diabetes)/{&Delta}age

//  #01 Description of sample

use hrs-gme-analysis2, clear
codebook diabetes female hsdegree age white bmi weight height, compact

//  #02 logit models

//  Mbmi: baseline model with bmi

qui logit diabetes c.bmi i.white c.age##c.age i.female i.hsdegree, or
estimates store Mbmi
mchange, amount(one sd)

//  Mwt: baseline model with height and weight

qui logit diabetes c.weight c.height ///
    i.white c.age##c.age i.female i.hsdegree, or
estimates store Mwt
mchange, amount(one sd)

//  compare models

estimates table Mbmi Mwt, stats(aic bic) eform  ///
    p(%9.3f) stfmt(%9.2f) b(%9.4f)

//  #04 Average DC for i.variable: ADC(white)

estimates restore Mbmi
margins, at(white=0) at(white=1) post
lincom _b[2._at] - _b[1._at]
mlincom 2-1, stats(all)

//  #05 Average DC for c.variable: ADC(bmi)

qui estimates restore Mbmi
qui sum bmi
local sd = r(sd)
margins, at(bmi=gen(bmi)) at(bmi=gen(bmi+`sd')) post
mlincom 2-1, stats(all)

//  #06 DC at mean for c.variable: DCM(bmi)

qui estimates restore Mbmi
mchange bmi, amount(sd) atmeans

//  margins

qui sum bmi
    local mn = r(mean) // change starts here
    local mn_plus = r(mean) + r(sd) // change ends here
margins, at(bmi=`mn') at(bmi=`mn_plus') atmeans

//  mtable

mtable, at(bmi=`mn') at(bmi=`mn_plus') atmeans post commands
mlincom 2-1

//  #07 proportional change: ADC(weight*1.14)

//  ADC(weight+25) saving predictions

qui estimates restore Mwt
mtable, gen(PRadd) ///
    at(weight=gen(weight)) at(weight=gen(weight+25)) post commands
mlincom 2-1

//  ADC(weight*1.14) saving predictions

qui estimates restore Mwt
mtable, gen(PRpct) ///
    at(weight=gen(weight)) at(weight=gen(weight*1.14)) post commands
mlincom 2-1

//  Compare ADC(weight*1.14) and ADC(weight+25)

generate  DCadd = PRadd2 - PRadd1
label var DCadd "DC(weight+25)"
sum DCadd // matches earlier ADC

generate DCpct = PRpct2 - PRpct1
lab var  DCpct "DC(weight*1.14)"
sum DCpct // matches earlier ADC

//  Histogram of effects

global TTLdcadd /// x
`"{&Delta}Pr(diabetes)/{&Delta}(weight{&rarr}weight+25)"'
global TTLdcaddS ///
`"{&Delta}Pr/{&Delta}(weight{&rarr}weight+25)"'
global TTLdcpct ///
`"{&Delta}Pr(diabetes)/{&Delta}(weight{&rarr}weight*1.14)"'
global TTLdcpctS ///
`"{&Delta}Pr/{&Delta}(weight{&rarr}weight*1.14)"'

sum DCadd
local mn = r(mean)
local graphname dcwt-add-hist
histogram DCadd, fcol(green*.8) lcol(green) width(.005) ///
    xtitle($TTLdcadd) /// xlabel(.0(.05).2) ///
    ylabel(0(5)20) ///
    xlabel(.0 0.0 .05 .05 `mn' "ADC" .1 .1 .15 .15 .2 .2)    ///
    scale(1.3) xsize(8) ysize(6.5) ///
    plotregion(margin(zero) lcol(white)) ///
    caption("`graphname' `tag'", siz(*.5) pos(5) col(gs10)) 
qui graph export `pgm'-`graphname'.emf, replace

sum DCpct
local mn = r(mean)
local graphname dcwt-pct-hist
histogram DCpct, fcol(blue*.8) lcol(blue) width(.005) ///
    xtitle($TTLdcpct) /// xlabel(.0(.05).2) ///
    ylabel(0(5)20) ///
    xlabel(.0 0.0 .05 .05 `mn' "ADC" .1 .1 .15 .15 .2 .2)    ///
    scale(1.3) xsize(8) ysize(6.5) ///
    plotregion(margin(zero) lcol(white)) ///
    caption("`graphname' `tag'", siz(*.5) pos(5) col(gs10)) 
qui graph export `pgm'-`graphname'.emf, replace

tabstat DCadd DCpct, stats(mean sd min max) format(%10.3f)

//  Test ADC(weight*1.14) = ADC(weight+25)

qui estimates restore Mwt
mtable, at(weight=gen(weight)) at(weight=gen(weight+25)) ///
        at(weight=gen(weight*1.14)) post commands
qui mlincom (2-1),         rowname(ADC add) clear
qui mlincom (3-1),         rowname(ADC pct) add
    mlincom (2-1) - (3-1), rowname(Difference) add dec(5)

//  #08 DC with linked variables: age and age-squared

//  ADC(age) using factor syntax

qui estimates restore Mbmi

mtable, at(age=gen(age)) at(age=gen(age+10)) post commands
mlincom 2-1, rowname(FV right) clear add

//  ADC(age) incorrect without factor variables

logit diabetes c.bmi i.white c.age c.agesq i.female i.hsdegree, or nolog
qui estimates store Mbmi_nofv

mtable, at(age=gen(age)) at(age=gen(age + 10)) post commands
mlincom 2-1, rowname(noFV wrong) add

//  ADC(age) correct without factor variables

qui estimates restore Mbmi_nofv
mtable, at(age=gen(age)    agesq=gen(agesq) ) ///
        at(age=gen(age+10) agesq=gen((age+10)^2)) post commands
mlincom 2-1, rowname(noFV right) add

//  #09 associated predictors: height and weight

//  Predict height from weight and weight squared

regress weight c.height##c.height, noci
scalar b0 = _b[_cons]
scalar b1 = _b[height]
scalar b2 = _b[c.height#c.height]

//  ADC(height+6, weight+predicted change)

qui estimates restore Mwt

mtable, post commands ///
    at(height=gen(height) weight=gen(weight))              /// observed
    at(height=gen(height+6)                                /// height + 6
       weight=gen(b0 + b1*(height+6) + b2*((height+6)^2))) //  -> weight
mlincom 2-1

//  #10 Why ADC(age) and DCM(age) are misleading

//  DC(age) for each observations using generate()

qui estimates restore Mbmi
margins, generate(PRage) at(age=gen(age)) at(age=gen(age + 10))

gen DCage10 = PRage2 - PRage1
lab var DCage10 "DC(age+10)"
sum DCage10

histogram DCage10, xtitle($TTLdpr_dage)
qui graph export `pgm'-dcage-dist.emf, replace

//  Probability against age with age-squared in model

qui estimates restore Mbmi
mgen, at(age=(55(2.5)100)) atmeans stub(PRD_)  replace // for line
lab var PRD_pr1  "Predicted probability"

twoway ///
    (connected PRD_pr1  PRD_age, lcol(red) lpat(solid) msym(i)) ///
   , ytitle(Pr(diabetes)) xtitle(Age) ///
    xlabel(55 55 60 60 65 65 70 70 75 75 77.5 "{&mu}" ///
        80 80 85 85 90 90 95 95 100 100)  ///
    xline(77.5, lcol(green*.5) lpat(dash) lwid(*1.2)) ///
    ylabel(0(.1).3, grid gmin gmax) ///
    title(Other variables held at means, position(11) size(*.7)) ///
    legend(off)
qui graph export `pgm'-prob-age.emf, replace

//  #11 comparing DCR(age) at different ages

qui estimates restore Mbmi

mtable, at(age=(60(10)90)) post atmeans commands
qui {
mlincom  2-1,      clear rowname(DCR60)
mlincom  3-2,        add rowname(DCR70)
mlincom  4-3,        add rowname(DCR80)
mlincom (2-1)-(3-2), add rowname(DCR60-DCR70)
mlincom (2-1)-(4-3), add rowname(DCR60-DCR80)
}
mlincom (3-2)-(4-3), add rowname(DCR70-DCR80) twidth(14)

//  #12 comparing effects within model

est restore Mbmi

//  are these equal

mchange bmi white, amount(sd)

//  compute effects simultaneously

qui sum bmi
local sd = r(sd)

margins, at(white=(0 1)) at(bmi=gen(bmi)) at(bmi=gen(bmi + `sd')) post
qui mlincom (2-1),         rowname(ADCwhite) clear
qui mlincom (4-3),         rowname(ADCbmi) add
    mlincom (2-1) + (4-3), rowname(Sum) add

//  #13 Comparing effects for subgroups

qui estimates restore Mbmi
qui sum bmi
local sd = r(sd)

//   Average prob for race subsamples

margins, over(white) at(bmi=gen(bmi)) at(bmi=gen(bmi+`sd')) post

//  Test if ADC(bmi) are equal for whites and non-whites

qui mlincom  4-2,        rowname(White ADC(bmi)) clear
qui mlincom  3-1,        rowname(Non-white ADC(bmi)) add
    mlincom (4-2)-(3-1), rowname(Difference) add

lincom (_b[2._at#1.white] - _b[1._at#1.white]) ///
     - (_b[2._at#0.white] - _b[1._at#0.white])

//  #14 Comparing ADC(female) across models

//  clone outcomes

clonevar  lhsbmi = diabetes
label var lhsbmi "TEMPORARY: clone diabetes for bmi equation"
clonevar  lhswt  = diabetes
label var lhswt "TEMPORARY: clone diabetes for height weight equation"

//  fit both models

gsem ///
    (lhsbmi <- c.bmi i.female i.white c.age##c.age i.hsdegree ///
        , logit) ///
    (lhswt  <- c.weight c.height i.female i.white c.age##c.age i.hsdegree ///
        , logit) ///
    , vce(robust)
est store Mgsem

//  ADC(female) for both models

margins, dydx(female) post

//  ADC(female|Mbmi)-ADC(female|Mwt)

mlincom 1-2, stats(all)

//  #15 Comparing groups

//  fit model with unconstrained parameters

logit diabetes ibn.white ///
    ibn.white#(i.female i.hsdegree c.age##c.age c.bmi), nocon or
qui estimates store Mgroups

//  Probability of diabetes by age and race

mgen, atmeans stub(PRw_) replace at(white=1 age=(55(2.5)100))
label var PRw_pr1 "White"
mgen, atmeans stub(PRn_) replace at(white=0 age=(55(2.5)100))
label var PRn_pr1 "Non-white"
mgen, dydx(white) atmeans stub(DCwn_) replace at(age=(55(2.5)100))
label var DCwn_ll1 "95% confidence interval"

twoway ///
    (connected PRw_pr1 PRw_age, clcol(blue) mcol(blue) msym(Oh)) ///
    (connected PRn_pr1 PRw_age, clcol(red) mcol(red) msym(Th)) ///
  , ytitle(Pr(diabetes)) xtitle(Age) ///
    xlabel(55(5)100) ylabel(0(.1).5, grid gmin gmax) ///
    legend(pos(12) ring(0) cols(2) order(2 1))
qui graph export `pgm'-prob-age-race.emf, replace

twoway ///
    (connected DCwn_d_pr1 DCwn_age, clcol(green) clpat(solid) msym(i)) ///
    (connected DCwn_ll1   DCwn_ul1 DCwn_age, ///
        clcol(brown brown) clpat(dash dash) msym(i i)) ///
  , ytitle("Pr(diabetes|white)-Pr(diabetes|non-white)", size(*.8)) ///
    xtitle(Age) ylabel(-.2(.1).1, grid gmax) xlabel(55(5)100) ///
    yline(0, lcol(black)) legend(pos(1) ring(0) cols(1) order(3)) ///

qui graph export `pgm'-dc-age-race.emf, replace

//  DCR(white) by age at means (plotted above)

mtable, dydx(white) at(age=(55(10)85)) stats(est p) atmeans commands

//  ADC(bmi) over race

mtable, over(white) at(bmi=gen(bmi)) at(bmi=gen(bmi+5)) post
qui mlincom  3-1,        rowname(ADC: nonwhite)   stats(est p) clear
qui mlincom  4-2,        rowname(ADC: white)      stats(est p) add
    mlincom (4-2)-(3-1), rowname(ADC: difference) stats(est p) add

//  DCR(age) by race

qui estimates restore Mgroups
mtable, at(age=55 white=(0 1)) at(age=65 white=(0 1)) atmeans post
qui mlincom  3-1,        rowname(DCR55: nonwhite) clear
qui mlincom  4-2,        rowname(DCR55: white) add
qui mlincom (4-2)-(3-1), rowname(DCR55: difference) add

qui estimates restore Mgroups
mtable, at(age=70 white=(0 1)) at(age=80 white=(0 1)) atmeans post
qui mlincom  3-1,        rowname(DCR70: nonwhite) add
qui mlincom  4-2,        rowname(DCR70: white) add
    mlincom (4-2)-(3-1), rowname(DCR70: difference) add

qui estimates restore Mgroups
mtable, at(age=85 white=(0 1)) at(age=95 white=(0 1)) atmeans post
qui mlincom  3-1,        rowname(DCR85: nonwhite) add
qui mlincom  4-2,        rowname(DCR85: white) add
    mlincom (4-2)-(3-1), rowname(DCR85: difference) add

//  #16 Decomposing compound variables: DC(weight) with bmi

//  bmi as (weight*703)/(height*height)

codebook scale703 height_inv, compact
logit diabetes c.scale703#c.weight#c.height_inv#c.height_inv ///
    i.white c.age##c.age i.female i.hsdegree, or nolog
estimates store Mbmifv

estimates table Mbmi Mbmifv, eform

//  ADC(weight)

qui estimates restore Mbmifv
mchange weight, amount(sd) delta(25)

qui estimates restore Mwt
mchange weight, amount(sd) delta(25)

//  gsem: compare ADC(weight) from Mbmi and Mwt

gsem ///
    (lhsbmi <- c.scale703#c.weight#c.height_inv#c.height_inv ///
        i.white c.age##c.age i.female i.hsdegree, logit) ///
    (lhswt  <- c.weight c.height i.female i.white c.age##c.age i.hsdegree ///
        , logit) ///
    , vce(robust)
est store Mgsem

margins, at(weight=gen(weight)) at(weight=gen(weight+25)) post

qui mlincom  2-1,          rowname(Mbmi ADC) clear
qui mlincom  4-3,          rowname(Mwt  ADC) add
    mlincom (4-3) - (2-1), rowname(Difference) add

log close
exit


DROP
qui {

global GRAPHsize scale(1.3) xsize(8) ysize(6.5) ///
    plotregion(margin(zero) lcol(white))

global LINsolidred  "msym(i) lcol(red)  lpat(solid)"
global LINsolidblue "msym(i) lcol(blue) lpat(solid)"
global LINdashblue  "msym(i) lcol(blue) lpat(dash)"
global LINdashred   "msym(i) lcol(red)  lpat(dash)"
global LINwht       clcol(blue) msym(Oh) mcol(blue) msiz(*1.5)
global LINnon       clcol(red)  msym(Sh) mcol(red)  msiz(*1.27)

global LINwht       msym(i) clcol(blue) msiz(*1.5)
global LINnon       msym(i) clcol(red)  msiz(*1.27)
global SYMwht       msym(Oh) mcol(blue) msiz(*1.5)
global SYMnon       msym(Sh) mcol(red)  msiz(*1.27)

global SYMnon       msym(Oh) mcol(red) msiz(*1.13)
global SYMwht       msym(Sh) mcol(blue)  msiz(*1.)

global SYMdc        mcol(green)  msym(Th) msiz(*1.1)
global LINdc        clcol(green)       clpat(solid)  msym(i)
global LINnonsig    clcol(green)       clpat(dash) msym(i)
global LINci        clcol(brown brown) clpat(dash dash) msym(i i) clwid(*.7 *.7)
global LINci        clcol(brown brown) clpat(solid solid) msym(i i) clwid(*.7 *.7)

global SYMohred  "msiz(*1.3) mlwid(*.9) msym(oh) mcol(red)"
global SYMshblue "msiz(*1.15) mlwid(*.9) msym(sh) mcol(blue)"

global TTLdpr_dage     "{&Delta}Pr(diabetes)/{&Delta}age"
global TTLpartial_dage "{&part}Pr(diabetes)/{&part}age"

}




/*DROP
twoway (scatter DCpct DCadd, mcol(red) msym(Oh)) , ///
    xtitle($TTLDCaddS) xlab(.0(.05).20, grid gmin gmax) ///
    ytitle($TTLdcpctS) ylab(.0(.05).20, grid gmin gmax) ///
    aspect(1) legend(off)
graph export eusmex2016-effects-scott-long-07-ADCadd-ADCpct.emf, replace
*/
