capture log close
log using mco18-nrm-nomocc-2018-03-27, replace text
version 14.2
clear all
macro drop _all
set linesize 80
set scheme s1manual

//  Modeling Categorical Outcomes - 2018
//  MNLM of occupation

local pgm mco18-nrm-nomocc-2018-03-27
local dte 2018-04-10
local who Scott Long
local tag "`pgm'.do `who' `dte'"

global graphfmt emf 
if "`c(os)'"!="Windows" global graphfmt pdf

//  #1+ mnlm with three categories ===========================================

//  #10 create data for binary logit as mnlm

use nomocc4, clear // load data
datasignature confirm // confirm data signature
codebook, compact // variable descriptions

tab occ, m // tabulation of occ variable

* original 5-cat dependent variable
mlogit occ i.white ed exper, base(1) nolog
estimates store full

* recode into categories labor, skilled, professional
clonevar occlsp = occ
recode   occlsp 5=3 1 2=1 3 4=2
lab def  occlsp 1 1Labor 2 2Skilled 3 3Prof
lab val  occlsp occlsp
tab occ occlsp, m // cross-tab new and old variable to ensure correct coding

* binary for labor(1) vs skilled (0), dropping professional
clonevar labrskil = occlsp
recode   labrskil 3=. 1=1 2=0
lab def  labrskil 1 1Labor 0 0Skilled
lab val  labrskil labrskil
tab occlsp labrskil, m

* binary for professional(1) vs skilled(0), dropping labor
clonevar profskil = occlsp
recode   profskil 1=. 3=1 2=0
lab def  profskil 1 1Prof 0 0Skilled
lab val  profskil profskil
tab occlsp profskil, m

* binary for labor(1) vs professional(0), dropping skilled
clonevar labrprof = occlsp
recode   labrprof 2=. 1=1 3=0
lab def  labrprof 1 1Labor 0 0Prof
lab val  labrprof labrprof
tab occlsp labrprof, m

//  #11 series of binary logits

quietly logit labrskil ed
estimates store labrskil
listcoef

quietly logit profskil ed
estimates store profskil
listcoef

quietly logit labrprof ed
estimates store labrprof
listcoef

estimates table labrprof labrskil profskil, stats(N) b(%9.3f) p(%6.2f)

//  #12 MNLM on three-category outcome

mlogit occlsp ed
listcoef

//  #21 descriptive statistics

tab1  occ, miss
codebook occ white ed exper, compact

//  #22 base 1

mlogit occ i.white ed exper, base(1) nolog
estimates store full
listcoef
listcoef white
listcoef white, pval(.05)

//  #23 base 2

mlogit occ i.white ed exper, base(2) nolog
listcoef
listcoef 1.white, pval(.05)

//  #24 wald tests of variables

quietly mlogit occ i.white ed exper, base(1)

* wald using test
test 1.white
test ed
test exper

* wald using mlogtest
mlogtest, wald

//  #25 LR tests of variables

estimates restore full

* drop white and conduct LR test
quietly mlogit occ ed exper, base(1)
estimates store dropwhite
lrtest full dropwhite // LR-test for white variable

* drop ed and conduct LR test
quietly mlogit occ i.white exper, base(1)
estimates store droped
lrtest full droped

* same results using mlogtest
estimates restore full
mlogtest, lr

//  #26 wald tests of combining categories

* using mlogtest
quietly mlogit occ i.white ed exper, base(5)
mlogtest, combine

* wald tests of combining categories using test with base 1
test [WhiteCol]

//  #27 lr tests of combining categories

quietly mlogit occ i.white ed exper, base(5)
mlogtest, lrcomb

//  #3+ mnlm with five categories | interpretation ===========================

//  #31 predicted probabilities

mlogit occ i.white ed exper, base(1) nolog
estimate store full

capture drop pr* // drops pr vars; capture error if variable does not exist

predict prmenial prblue prcraft prwhite prprof
label var prmenial "Pr(menial)"
label var prblue "Pr(blue)"
label var prcraft "Pr(craft)"
label var prwhite "Pr(white)"
label var prprof "Pr(prof)"

local graphname "phat-dotplot"
dotplot prmenial prblue prcraft prwhite prprof, ///
    ylab(0(.25)1, grid gmin gmax) mcolor(blue)  ///
    caption("`graphname' `tag'",size(*.5) pos(5) col(gs10))
graph export `pgm'-`graphname'.$graphfmt, replace

* list of high and low probabilities
sort prprof
list prprof occ white ed exper if prprof<.05, clean
list prprof occ white ed exper if prprof>.90, clean

//  #32 AME and MEM (average marginal effects and marginal effects @ mean)

estimate restore full

* AME
mchange, amount(one sd)
local graphname "#32mchangeplot-ame"
mchangeplot, amount(sd sd) min(-.3) max(.4) gap(.1) mcol(rainbow) aspect(.3) ///
    title(Average discrete change, position(11)) leftmargin(4) ///
    caption("`graphname' `tag'",size(*.5) pos(5) col(gs10))
graph export `pgm'-`graphname'.$graphfmt, replace

local graphname "#32mchangeplot-ame-sig10"
mchangeplot, amount(sd sd) min(-.3) max(.4) gap(.1) mcol(rainbow) aspect(.3) ///
    title(Average discrete change, position(11)) leftmargin(4) sig(.10) ///
    caption("`graphname' `tag'",size(*.5) pos(5) col(gs10))
graph export `pgm'-`graphname'.$graphfmt, replace

* MEM
mchange, atmeans amount(one sd)
local graphname "#32mchangeplot-mem"
mchangeplot, amount(sd sd) min(-.3) max(.4) gap(.1) mcol(rainbow) aspect(.3) ///
    title(Discrete change at the mean, position(11)) leftmargin(4) ///
    caption("`graphname' `tag'",size(*.5) pos(5) col(gs10))
graph export `pgm'-`graphname'.$graphfmt, replace

* AME and MEM  from to for white
mchange 1.white, stats(from to change p)
mchange 1.white, stats(from to change p) atmeans

//  #33 predictions by race as observed

mtable if white==1, rowname(White) clear nobs
mtable if white==0, rowname(Nonwhite) below nobs

//  #34 predictions by race with counterfactual for white

mtable, at(white=1) rowname(White) clear nobs // all treated as white
mtable, at(white=0) rowname(Nonwhite) below nobs // all treated as nonwhite
mtable, dydx(white) below nobs stat(est p) // ADC for white

//  #35 predictions with other vars at local means

mtable if white==1, rowname(White) clear nobs atmeans
mtable if white==0, rowname(Nonwhite) below nobs atmeans

//  #36 predictions with other variables at global means

mtable, at(white=1) rowname(White) clear nobs atmeans
mtable, at(white=0) rowname(Nonwhite) below nobs atmeans
mtable, dydx(white) below nobs stat(est p) atmeans

//  #4+ mnlm odds ratios and OR plots ========================================

mlogit occ i.white ed exper, nolog

//  #41 odds ratios

//  all variables

listcoef

//  white only with options

listcoef 1.white
listcoef 1.white, adj // only adjacent contrasts
listcoef 1.white, adj gt // only lower/higher contrasts
listcoef 1.white, adj lt // only higher/lower contrasts

//  only significant contrasts

listcoef, gt pvalue(.05) // 1st gt 2nd; sig at .05
listcoef, adj lt pvalue(.05) // 1st lt 2nd, adjacent, sig at .05

//  #42 w/o sig levels

local graphname "mlogitplot-nosig"
mlogitplot 1.white ed exper, amount(sd sd) base(5) linep(1) min(-2.75) max(.55) ///
    packed ntics(7) aspect(.5) mcolors(rainbow) varlabel leftmargin(4) ///
    caption("`graphname' `tag'",size(*.5) pos(5) col(gs10))
graph export `pgm'-`graphname'.$graphfmt, replace

//  #43 OR with significance levels

local graphname "mlogitplot-sig-baseP5"
mlogitplot 1.white ed exper, linep(.1) amount(sd sd) base(5) leftmargin(4) ///
    min(-3) max(.5) ntics(8) aspect(.5) mcol(rainbow) linegapfactor(.6) ///
    caption("`graphname' `tag'",size(*.5) pos(5) col(gs10))
graph export `pgm'-`graphname'.$graphfmt, replace

//  #44 OR with significance levels - base M

local graphname "mlogitplot-sig-baseM1"
mlogitplot 1.white ed exper, linep(.1) amount(sd sd) base(1) leftmargin(4) ///
    min(-.5) max(3) ntics(8) aspect(.5) mcol(rainbow) linegapfactor(.6) ///
    caption("`graphname' `tag'",size(*.5) pos(5) col(gs10))
graph export `pgm'-`graphname'.$graphfmt, replace

//  #45 OR with significance levels with offset

local graphname "mlogitplot-sig-baseP5-offset"
mlogitplot 1.white ed exper, linep(.1) amount(sd sd) base(5) leftmargin(4) ///
    min(-3) max(.5) ntics(8) aspect(.5) mcol(rainbow) linegapfactor(.6) ///
    offsetlist(0 3 0 -3 0    2 -2 -2 0 0    1 -1 -3 0 3 ) ///
    caption("`graphname' `tag'",size(*.5) pos(5) col(gs10))
graph export `pgm'-`graphname'.$graphfmt, replace

//  #46 OR + DC plots with AME

mchange
local graphname "ormchangeplot-ame-baseP5-offset"
mlogitplot 1.white ed exper, linep(.1) amount(sd sd) base(1) mchange ///
    min(-.5) max(2.5) ntics(7) aspect(.5) mcol(rainbow) ///
    offsetlist(0 3 0 -3 0    2 -2 -2 0 0    1 -1 -3 0 3 ) ///
    linegapfactor(.5) leftmargin(4) /// msizefactor(1.5)
    caption("`graphname' `tag'",size(*.5) pos(5) col(gs10))
graph export `pgm'-`graphname'.$graphfmt, replace

//  #47 OR with two levels of significance

local graphname "mlogitplot-siglevels"
mlogitplot 1.white ed exper, amount(sd sd) base(5) min(-3) max(.5) gap(.5) ///
    aspect(.5) mcolors(rainbow) linegapfactor(.6)  /// varlabel
    lcolor(red) lshades linep(.1 .2 .3 .4) ///
    subtitle("p-values .1 (light) to .4 (dark)", position(11)) ///
    caption("`graphname' `tag'",size(*.5) pos(5) col(gs10))
graph export `pgm'-`graphname'.$graphfmt, replace

//  #5+ mnlm iia test ========================================================

//  #51 testing iia

mlogit occ i.white ed exper, nolog

set seed 14293 // needed for Small Hsia test
mlogtest, iia

set seed 201193 // needed for Small Hsia test
mlogtest, iia

set seed 1102 // needed for Small Hsia test
mlogtest, iia

log close
exit
